==> mongoop-0.1.0.tgz: package/index.js <==
/*
 *  mongoop
 *  Copyright(c) 2017 Giles Aamoth
 *  MIT Licensed
*/
'use strict';

var mongoose = require('mongoose');

module.exports = mongoop

function mongoop(uri) {
  // connection to mongodb database
  let connection = mongoose.connect(uri)

  // handle any errors from trying to connect to database
  let errorhandler = connection.connection.on('error', err => console.error(err))

  // object that will be returned by function
  // expose private functions through object methods
  let mongoopObject = {
    collections: collections,
    sethooks: sethooks,
    prehook: prehook,
    posthook: posthook,
    setplugins: setplugins
  }

  // create a property for each desired collection
  // onto the object returned by mongoop function
  // this exposes each collections database operations
  // and various methods to setup how collections behave
  function collections(options) {
    if (typeof options === 'object') { // a single collection
      let query = setup(options)
      let opers = operations(query.query)
      init(options.model, opers, query)
    }
    if (typeof options === 'array') { // multiple collections
      options.forEach(e => {
        let query = setup(e)
        let opers = operations(query.query)
        init(e.model, opers, query)
      })
    }
  }

  // setup collection with model name and schema
  function setup(collection) {
    let queryschema = connection.Schema(collection.schema)
    let query = connection.model(collection.model, queryschema)
    return {
      queryschema: queryschema,
      query: query
    }
  }

  // put property onto mongoop functions return object
  // given the name of the collection
  // put method on this property for access to db operations
  // for collection, access to schema and query
  function init(model, operations, queryobject) {
    mongoopObject[model] = {
      schema: queryobject.queryschema,
      query: queryobject.query,
      operations: operations
    }
  }

  // returns object containing CRUD operations on a collection
  function operations(Query) {
    return {
      create (options, callback) {
        let query = new Query(options)

        query.save(err => {
          if (err) {
            return callback(err)
          }
          callback(null, options)
        })
      },
      read (querymethod, options, callback) {
        Query[querymethod](options, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      update (query, update, callback) {
        Query.update(query, update, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      delete (query, callback) {
        Query.remove(query, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      }
    }
  }

  // accepts single hook assignments (object)
  // accepts multiple hook assignments (array)
  // assigns hooks onto collection schema
  //// TODO: add functionality for running hooks in series or parallel
  function sethooks(hooks) {
    if (typeof hooks === 'object') { // a single hook
      hooks[1].type === 'pre'
        ? prehook(hooks[0], hooks[1], hooks[2])
        : posthook(hooks[0], hooks[1], hooks[2])
    }
    if (typeof hooks === 'array') { // multiple hooks
      param.forEach(e => {
        e[1].type === 'pre'
          ? prehook(e[0], e[1], e[2])
          : posthook(e[0], e[1], e[2])
      })
    }
  }

  // assign a prehook to collection
  function prehook(collection, action, callback) {
    let schema = mongoopObject[collection].schema
    schema.pre(action, callback)
  }

  // assign a posthook to collection
  function posthook(collection, action, callback) {
    let schema = mongoopObject[collection].schema
    schema.post(action, callback)
  }

  // setplugins -- noop
  function setplugins() {}

  return mongoopObject
}

==> mongoop-0.1.1.tgz: package/index.js <==
/*
 *  mongoop
 *  Copyright(c) 2017 Giles Aamoth
 *  MIT Licensed
*/
'use strict';

var mongoose = require('mongoose');

module.exports = mongoop

function mongoop(uri) {
  // connection to mongodb database
  let connection = mongoose.connect(uri)

  // handle any errors from trying to connect to database
  let errorhandler = connection.connection.on('error', err => console.error(err))

  // object that will be returned by function
  // expose private functions through object methods
  let mongoopObject = {
    collections: collections,
    sethooks: sethooks,
    prehook: prehook,
    posthook: posthook,
    setplugins: setplugins
  }

  // create a property for each desired collection
  // onto the object returned by mongoop function
  // this exposes each collections database operations
  // and various methods to setup how collections behave
  function collections(options) {
    if (typeof options === 'object') { // a single collection
      let query = setup(options)
      let opers = operations(query.query)
      init(options.model, opers, query)
    }
    if (typeof options === 'array') { // multiple collections
      options.forEach(e => {
        let query = setup(e)
        let opers = operations(query.query)
        init(e.model, opers, query)
      })
    }
  }

  // setup collection with model name and schema
  function setup(collection) {
    let queryschema = connection.Schema(collection.schema)
    let query = connection.model(collection.model, queryschema)
    return {
      queryschema: queryschema,
      query: query
    }
  }

  // put property onto mongoop functions return object
  // given the name of the collection
  // put method on this property for access to db operations
  // for collection, access to schema and query
  function init(model, operations, queryobject) {
    mongoopObject[model] = {
      schema: queryobject.queryschema,
      query: queryobject.query,
      operations: operations
    }
  }

  // returns object containing CRUD operations on a collection
  function operations(Query) {
    return {
      create (options, callback) {
        let query = new Query(options)

        query.save(err => {
          if (err) {
            return callback(err)
          }
          callback(null, options)
        })
      },
      read (querymethod, options, callback) {
        Query[querymethod](options, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      update (query, update, callback) {
        Query.update(query, update, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      },
      delete (query, callback) {
        Query.remove(query, (err, returnValue) => {
          if (err) {
            return callback(err)
          }
          callback(null, returnValue)
        })
      }
    }
  }

  // accepts single hook assignments (array)
  // accepts multiple hook assignments (array of arrays)
  // assigns hooks onto collection schema
  //// TODO: add functionality for running hooks in series or parallel
  function sethooks(hooks) {
    hooks.forEach(e => {
      e[1].type === 'pre'
        ? prehook(e[0], e[1], e[2])
        : posthook(e[0], e[1], e[2])
    })
  }

  // assign a prehook to collection
  function prehook(collection, action, callback) {
    let schema = mongoopObject[collection].schema
    schema.pre(action, callback)
  }

  // assign a posthook to collection
  function posthook(collection, action, callback) {
    let schema = mongoopObject[collection].schema
    schema.post(action, callback)
  }

  // setplugins -- noop
  function setplugins() {}

  return mongoopObject
}
